//****************************************************************************
//                                                 Liu-Yang Lorenz-like system
//
//                                          Les Monts du Roumois, May 12, 2023
//****************************************************************************
# include <cstdlib>
# include <iostream>
# include <fstream>
# include <iomanip>
# include <cmath>
# include <ctime>

using namespace std;

//const double a = 1.0;
//const double b = 15.00;                                    // Original value
//const double b = 12.158;               // After the merging attractor crisis
//const double b = 12.1574;             // Before the merging attractor crisis
// ---------------------------------------------------------------------------
  const double a = 2.0;                              // Burke & Shaw attractor
  const double b = 25.18;                           
  const double c = 0.05;
// ----------------------------- Burke & Shaw attractor ----------------------
//const double a = 8.0;                              
//const double b = 50.00;                           
//const double c = 0.15;
  const double dt = 0.05;        
// ----------------------------- Burke & Shaw attractor ----------------------
//const double a = 15.7975;
//const double b = 40.00;                           
//const double c = 1.00;
//const double dt = 0.01;        

  const int m = 3;                                    // state space dimension 
// **************** Liu & Yang system ****************************************
#define fx(x,y,z) - a*x + a*y
#define fy(x,y,z) b*x - x*z
#define fz(x,y,z) - c*z + x*y

void Runge(double *v);                             // Fourth-order Runge-Kutta

int main ( )
{
  int i, j = 0, k = 0;
  const int Ntrans = 90000;
//const int Npt = 56000;                           // After the merging crisis
  const int Npt = 10000;                      // number of data points 
  const double zp = b;
  double *v, *sv, xp, sxp; 
  double Xmin1 = +1.0E+5;
  double Xmax1 = -1.0E+5;
  double Xmin2 = +1.0E+5;
  double Xmax2 = -1.0E+5;

  v = new double[m];
  sv = new double[m];
//****************************************************************************
  cout << " a = " << a << " b = " << b << " c = " << c << "\n";
  ofstream data_12 ("liuyangmap.dat");

  v[0] = 0.1;
  v[1] =-0.1;
  v[2] = 0.1;
  do
  {
    for (i = 0; i < m; i++)
    {
      sv[i] = v[i];
    }
    j++;
    Runge(v);
    if (j > Ntrans && v[2] > zp && sv[2] <= zp)
    {
      k++;
      xp = v[0] - ((v[2] - zp)/(v[2] - sv[2]))*(v[0] - sv[0]);
      if (xp < 0.0)
      {
	Xmin1 = min(Xmin1,xp);
	Xmax1 = max(Xmax1,xp);
      }
      else
      {
	Xmin2 = min(Xmin2,xp);
	Xmax2 = max(Xmax2,xp);
      }
    }
  } while (k < Npt);
  cout << Xmin1 << " < x_n < " << Xmax1 << "\n";
  cout << Xmin2 << " < x_n < " << Xmax2 << "\n";
  k = 0;
  do
  {
    for (i = 0; i < m; i++)
    {
      sv[i] = v[i];
    }
    Runge(v);
    if (v[2] > zp && sv[2] <= zp)
    {
      k++;
      if (k % 1000 == 0) cout << k << "\n";
      xp = v[0] - ((v[2] - zp)/(v[2] - sv[2]))*(v[0] - sv[0]);
      if (xp < 0.0)
      {
	xp = - (xp - Xmin1)/(Xmax1 - Xmin1);
        if (k > 1) data_12 << sxp << " " << xp << "\n";
        sxp = xp;
      }
      else
      {
	xp = (xp - Xmin2)/(Xmax2 - Xmin2);
        if (k > 1) data_12 << sxp << " " << xp << "\n";
        sxp = xp;
      }
    }
  } while (k < Npt);
  data_12.close ( );
  system("xmgrace liuyangmap.dat -param liuyangmap.par"); 
  return 0;
}
// ***************************************************************************
   void Runge(double *v)                           // Fourth-order Runge-Kutta
// ***************************************************************************
   {
      int j;
      double phi[m], savev[m], FF[m];

      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        savev[j]=v[j];
        phi[j]=FF[j];
        v[j]=savev[j]+0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j]=savev[j] + 0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j] = savev[j] + dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        v[j] = savev[j] + (phi[j] + FF[j])*dt/6.0;
      }
   }
// ***************************************************************************
